-- Settings
StatusBars2_Settings = { };

-- Bars
local bars = {};

-- Bar spacing
local kGroupSpacing = 18;

-- Power types
local kMana = 0;
local kRage = 1;
local kFocus = 2;
local kEnergy = 3;
local kRunicPower = 6;

-- Druid forms
local kCasterForm = 0;
local kBearForm = 1;
local kCatForm = 2;

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_OnLoad
--
--  Description:    Main frame OnLoad handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_OnLoad( self )

    -- Create bars
    StatusBars2_CreateBars( );
        
    -- Register for events
    self:SetScript( "OnEvent", StatusBars2_OnEvent );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD" );
    self:RegisterEvent( "UPDATE_SHAPESHIFT_FORM" );
    
    -- Print a status message
    StatusBars2_Trace( 'StatusBars 2 initialized' );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_OnEvent
--
--  Description:    Main frame event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_OnEvent( self, event )

    -- Player entering world, initialize the addon
    if( event == "PLAYER_ENTERING_WORLD" ) then
    
        -- Set default options
        StatusBars2_SetDefaultOptions( );
        
        -- Update bar visibility and location
        StatusBars2_UpdateBars( );
        
        -- Initialize the option panel controls
        StatusBars2_Options_DoDataExchange( false );
    
    -- Druid change form
    elseif( event == "UPDATE_SHAPESHIFT_FORM" ) then
    
        StatusBars2_UpdateBars( );
    
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateBars
--
--  Description:    Create all the bars
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateBars( )

    -- Player bars
    StatusBars2_CreateHealthBar( "StatusBars2_HealthBar", "player" );
    StatusBars2_CreatePowerBar( "StatusBars2_RageBar", "player", kRage );
    StatusBars2_CreatePowerBar( "StatusBars2_EnergyBar", "player", kEnergy );
    StatusBars2_CreatePowerBar( "StatusBars2_ManaBar", "player", kMana );
    StatusBars2_CreatePowerBar( "StatusBars2_RunicPowerBar", "player", kRunicPower );
    StatusBars2_CreateAuraBar( "StatusBars2_AuraBar", "player" );
    
    -- Target bars
    StatusBars2_CreateHealthBar( "StatusBars2_TargetHealthBar", "target" );
    StatusBars2_CreatePowerBar( "StatusBars2_TargetPowerBar", "target", nil );
    StatusBars2_CreateAuraBar( "StatusBars2_TargetAuraBar", "target" );
    
    -- Pet bars
    StatusBars2_CreateHealthBar( "StatusBars2_PetHealthBar", "pet" );
    StatusBars2_CreatePowerBar( "StatusBars2_PetFocusBar", "pet", kFocus );
    StatusBars2_CreatePowerBar( "StatusBars2_PetManaBar", "pet", kMana );
    StatusBars2_CreatePowerBar( "StatusBars2_PetEnergyBar", "pet", kEnergy );
    StatusBars2_CreateAuraBar( "StatusBars2_PetAuraBar", "pet" );
    
    -- Specialty bars
    StatusBars2_CreateComboBar( "StatusBars2_ComboBar" );
    StatusBars2_CreateRuneBar( "StatusBars2_RuneBar" );
    StatusBars2_CreateAuraStackBar( "StatusBars2_SunderBar", "Sunder Armor", "target", 5, 1, 0.5, 0 );
    StatusBars2_CreateAuraStackBar( "StatusBars2_DeadlyPoisonBar", "Deadly Poison", "target", 5, 0, 1, 0 );
    StatusBars2_CreateAuraStackBar( "StatusBars2_MaelstromWeaponBar", "Maelstrom Weapon", "player", 5, 1, 0, 1 );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdateBars
--
--  Description:    Update bar visibility and location
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdateBars( )

    -- Hide the bars
    for name, bar in pairs( bars ) do
        StatusBars2_HideBar( bar );
    end
    
    -- Get the current class and form
    local class = UnitClass( "player" );
    local form = GetShapeshiftForm( true );
   
    -- Player health
    if( StatusBars2_Settings[ "ShowPlayerHealth" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_HealthBar" ], 1, 1 );
    end
    
    -- Player power
    if( StatusBars2_Settings[ "ShowPlayerPower" ] == true ) then
    
        -- Mana
        if( class == "Hunter" or class == "Mage" or class == "Priest" or class == "Paladin" or class == "Shaman" or class == "Warlock" or ( class == "Druid" and ( form == kCasterForm or StatusBars2_Settings[ "ShowDruidMana" ] == true ) ) ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_ManaBar" ], 1, 2 );
        end
    
        -- Rage
        if( class == "Warrior" or ( class == "Druid" and form == kBearForm ) ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_RageBar" ], 1, 3 );
        end
        
        -- Energy
        if( class == "Rogue" or ( class == "Druid" and form == kCatForm ) ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_EnergyBar" ], 1, 4 );
        end
        
        -- Runic power
        if( class == "Death Knight" ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_RunicPowerBar" ], 1, 5 );
        end
    end
        
    -- Combo points
    if( StatusBars2_Settings[ "ShowComboPoints" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_ComboBar" ], 1, 5 );
    end
    
    -- Runes
    if( StatusBars2_Settings[ "ShowRunes" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_RuneBar" ], 1, 6 );
    end
    
    -- Shaman Maelstrom Weapon
    if( class == "Shaman" and StatusBars2_Settings[ "ShowMaelstromWeapon" ] == true) then
        StatusBars2_EnableBar( bars[ "StatusBars2_MaelstromWeaponBar" ], 1, 7 );
    end
    
    -- Sunder armor
    if( StatusBars2_Settings[ "ShowSunderArmor" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_SunderBar" ], 1, 8 );
    end
    
    -- Deadly poison
    if( StatusBars2_Settings[ "ShowDeadlyPoison" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_DeadlyPoisonBar" ], 1, 9 );
    end
    
    -- Player araus
    if( StatusBars2_Settings[ "ShowPlayerBuffs" ] == true or StatusBars2_Settings[ "ShowPlayerDebuffs" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_AuraBar" ], 1, 10 );
        bars[ "StatusBars2_AuraBar" ]:SetHeight( StatusBars2_Settings[ "AuraSize" ] );
    end
    
    -- Target health
    if( StatusBars2_Settings[ "ShowTargetHealth" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_TargetHealthBar" ], 2, 1 );
    end
    
    -- Target power
    if( StatusBars2_Settings[ "ShowTargetPower" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_TargetPowerBar" ], 2, 2, true );
    end;
    
    -- Target auras
    if( StatusBars2_Settings[ "ShowTargetBuffs" ] == true or StatusBars2_Settings[ "ShowTargetDebuffs" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_TargetAuraBar" ], 2, 3 );
        bars[ "StatusBars2_TargetAuraBar" ]:SetHeight( StatusBars2_Settings[ "AuraSize" ] );
    end
    
    -- Pet health
    if( ( class == "Hunter" or class == "Warlock" or class == "Death Knight" ) and StatusBars2_Settings[ "ShowPetHealth" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_PetHealthBar" ], 3, 1 );
    end
    
    -- Pet power
    if( StatusBars2_Settings[ "ShowPetPower" ] == true ) then
        if( class == "Hunter" ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_PetFocusBar" ], 3, 2 );
        elseif( class == "Warlock" ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_PetManaBar" ], 3, 3 );
        elseif( class == "Death Knight" ) then
            StatusBars2_EnableBar( bars[ "StatusBars2_PetEnergyBar" ], 3, 4 );
        end
    end
    
    -- Pet araus
    if( StatusBars2_Settings[ "ShowPetBuffs" ] == true or StatusBars2_Settings[ "ShowPetDebuffs" ] == true ) then
        StatusBars2_EnableBar( bars[ "StatusBars2_PetAuraBar" ], 3, 4 );
        bars[ "StatusBars2_PetAuraBar" ]:SetHeight( StatusBars2_Settings[ "AuraSize" ] );
    end
    
    -- If not locked drag on a left click
    if( StatusBars2_Settings[ "Locked" ] ~= true ) then
        StatusBars2:RegisterForDrag( "LeftButton" );
    else
        StatusBars2:RegisterForDrag( );
    end
    
    -- Update the layout
    StatusBars2_UpdateLayout( );
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_EnableBar
--
--  Description:    Enable a status bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_EnableBar( bar, group, index, removeWhenHidden )

    -- Set the layout properties
    bar.group = group;
    bar.index = index;
    bar.removeWhenHidden = removeWhenHidden;
    
    -- Initialize the incombat flag
    local inCombat = UnitAffectingCombat( "player" );
    bar.inCombat = inCombat ~= nil;
    
    -- Enable the event and update handlers
    bar:SetScript( "OnEvent", bar.OnEvent );
    bar:SetScript( "OnUpdate", bar.OnUpdate );
    
    -- Notify the bar is is enabled
    bar:OnEnable( );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_HideBar
--
--  Description:    Hide a status bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_HideBar( bar )

    -- Remove the event and update handlers
    bar:SetScript( "OnEvent", nil );
    bar:SetScript( "OnUpdate", nil );
    
    -- Clear the layout properties
    bar.group = nil;
    bar.index = nil;
    bar.removeWhenHidden = nil;

    -- Hide the bar
    bar:Hide( );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdateLayout
--
--  Description:    Update the layout of the bars
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdateLayout( )

    local layoutBars = {}

    -- Build a list of bars to layout
    for name, bar in pairs( bars ) do
    
        -- If the bar has a group and index set include it in the layout
        if( bar.group ~= nil and bar.index ~= nil and ( bar.removeWhenHidden == nil or bar:IsVisible( ) == 1 ) ) then
            table.insert( layoutBars, bar );
        end
    end
    
    -- Order the bars
    table.sort( layoutBars, StatusBars2_BarCompareFunction );
    
    -- Lay them out
    local group = nil;
    local offset = 0;
    for i, bar in ipairs( layoutBars ) do
    
        -- Add space between groups
        if( group ~= nil and group ~= bar.group ) then
            offset = offset - kGroupSpacing;
        end
        group = bar.group;
        
        -- Position the bar
        bar:SetPoint( "TOPLEFT", StatusBars2, "TOPLEFT", 0, offset );
        
        -- Update the offset
        offset = offset - ( bar:GetHeight( ) - 2 );
    
    end

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_BarCompareFunction
--
--  Description:    Function for comparing two bars
--
-------------------------------------------------------------------------------
--
function StatusBars2_BarCompareFunction( bar1, bar2 )

    return bar1.group < bar2.group or ( bar1.group == bar2.group and bar1.index < bar2.index );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateHealthBar
--
--  Description:    Create a health bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateHealthBar( name, unit )

    -- Create the bar
    local bar = StatusBars2_CreateContinuousBar( name, unit, 1, 0, 0 );
    
    -- Set the event handlers
    bar.OnEvent = StatusBars2_HealthBar_OnEvent;
    bar.OnEnable = StatusBars2_HealthBar_OnEnable;
    bar.BarIsVisible = StatusBars2_HealthBar_IsVisible;
    
    -- Register for events
    bar:RegisterEvent( "UNIT_HEALTH" );
    bar:RegisterEvent( "UNIT_MAXHEALTH" );
    bar:RegisterEvent( "PLAYER_REGEN_DISABLED" );
    bar:RegisterEvent( "PLAYER_REGEN_ENABLED" );
    if( unit == "target" ) then
        bar:RegisterEvent( "PLAYER_TARGET_CHANGED" );
    elseif( unit == "pet" ) then
        bar:RegisterEvent( "UNIT_PET" );
    end

    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreatePowerBar
--
--  Description:    Create a power bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreatePowerBar( name, unit, powerType )

    -- Create the power bar
    local bar = StatusBars2_CreateContinuousBar( name, unit, 1, 1, 0 );
    
    -- Save the power type
    bar.powerType = powerType;
    
    -- Set the color
    StatusBars2_SetPowerBarColor( bar );
    
    -- Set the event handlers
    bar.OnEvent = StatusBars2_PowerBar_OnEvent;
    bar.OnUpdate = StatusBars2_PowerBar_OnUpdate;
    bar.OnEnable = StatusBars2_PowerBar_OnEnable;
    bar.BarIsVisible = StatusBars2_PowerBar_IsVisible;
    
    -- Register for events
    bar:RegisterEvent( "PLAYER_REGEN_DISABLED" );
    bar:RegisterEvent( "PLAYER_REGEN_ENABLED" );
    if( powerType == kEnergy or powerType == nil ) then
        bar:RegisterEvent( "UNIT_ENERGY" );
        bar:RegisterEvent( "UNIT_MAXENERGY" );
    elseif( powerType == kRage or powerType == nil ) then
        bar:RegisterEvent( "UNIT_RAGE" );
        bar:RegisterEvent( "UNIT_MAXRAGE" );
    elseif( powerType == kMana or powerType == nil ) then
        bar:RegisterEvent( "UNIT_MANA" );
        bar:RegisterEvent( "UNIT_MAXMANA" );
    elseif( powerType == kFocus or powerType == nil ) then
        bar:RegisterEvent( "UNIT_FOCUS" );
        bar:RegisterEvent( "UNIT_MAXFOCUS" );
    elseif( powerType == kRunicPower or powerType == nil ) then
        bar:RegisterEvent( "UNIT_RUNIC_POWER" );
        bar:RegisterEvent( "UNIT_MAXRUNIC_POWER" );
    end
    if( unit == "target" ) then
        bar:RegisterEvent( "PLAYER_TARGET_CHANGED" );
        bar:RegisterEvent( "UNIT_SPELLCAST_START" );
        bar:RegisterEvent( "UNIT_SPELLCAST_STOP" );
        bar:RegisterEvent( "UNIT_SPELLCAST_FAILED" );
        bar:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED" );
        bar:RegisterEvent( "UNIT_SPELLCAST_DELAYED" );
        bar:RegisterEvent( "UNIT_SPELLCAST_CHANNEL_START" );
        bar:RegisterEvent( "UNIT_SPELLCAST_CHANNEL_UPDATE" );
        bar:RegisterEvent( "UNIT_SPELLCAST_CHANNEL_STOP" );
    elseif( unit == "pet" ) then
        bar:RegisterEvent( "UNIT_PET" );
    end
    if( powerType == nil ) then
        bar:RegisterEvent( "UNIT_MODEL_CHANGED" );
    end
    
    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateAuraStackBar
--
--  Description:    Create bar to track the stack size of a buff or debuff
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateAuraStackBar( name, aura, unit, count, r, g, b )

    -- Create the bar
    local bar = StatusBars2_CreateDiscreteBar( name, unit, count, r, g, b );
    
    -- Save the aura name and unit
    bar.aura = aura;
    bar.unit = unit;
    
    -- Set the event handlers
    bar.OnEvent = StatusBars2_AuraStackBar_OnEvent;
    
    -- Register for events
    bar:RegisterEvent( "PLAYER_TARGET_CHANGED" );
    bar:RegisterEvent( "PLAYER_REGEN_ENABLED" );
    bar:RegisterEvent( "PLAYER_REGEN_DISABLED" );
    bar:RegisterEvent( "COMBAT_LOG_EVENT_UNFILTERED" );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateComboBar
--
--  Description:    Create a combo point bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateComboBar( name )

    -- Create the bar
    local bar = StatusBars2_CreateDiscreteBar( name, "player", MAX_COMBO_POINTS, 1, 0, 0 );
    
    -- Set the event handlers
    bar.OnEvent = StatusBars2_ComboBar_OnEvent;
    bar.OnEnable = StatusBars2_ComboBar_OnEnable;
    
    -- Register for events
    bar:RegisterEvent( "PLAYER_TARGET_CHANGED" );
    bar:RegisterEvent( "UNIT_COMBO_POINTS" );
    bar:RegisterEvent( "PLAYER_REGEN_ENABLED" );
    bar:RegisterEvent( "PLAYER_REGEN_DISABLED" );
    
    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateRuneBar
--
--  Description:    Create a rune bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateRuneBar( name )

    -- Create the bar
    local bar = StatusBars2_CreateBar( name, "player", "StatusBars2_RuneFrameTemplate" );
    
    -- Create the rune table
    bar.runes = {};
    
    -- Initialize the rune buttons
    local i;
    for i = 1, 6 do
        local rune = getglobal( name .. '_RuneButton' .. i );
        RuneFrame_AddRune( bar, rune );
        rune.rune = getglobal( rune:GetName( ) .. "Rune" );
        rune.fill = getglobal( rune:GetName( ) .. "Fill" );
        rune.shine = getglobal( rune:GetName( ) .. "ShineTexture" );
        RuneButton_Update( rune );
    end
    
    -- Swap frost and unholy runes
    RuneFrame_FixRunes( bar );
    
    -- Set the event handlers
    bar.OnEvent = StatusBars2_RuneBar_OnEvent;
    bar.OnEnable = StatusBars2_RuneBar_OnEnable;
    
    -- Register for events
    bar:RegisterEvent( "RUNE_POWER_UPDATE" );
    bar:RegisterEvent( "RUNE_TYPE_UPDATE" );
    bar:RegisterEvent( "PLAYER_REGEN_ENABLED" );
    bar:RegisterEvent( "PLAYER_REGEN_DISABLED" );
    
    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateContinuousBar
--
--  Description:    Create a bar to display a range of values
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateContinuousBar( name, unit, r, g, b )

    -- Create the bar
    local bar = StatusBars2_CreateBar( name, unit, "StatusBars2_ContinuousBarTemplate" );
    
    -- Set the background color
    bar:SetBackdropColor( 0, 0, 0, 0.35 );
    
    -- Get the status and text frames
    bar.status = getglobal( name .. "_Status" );
    bar.text = getglobal( name .. "_Text" );
    bar.percentText = getglobal( name .. "_PercentText" );
    bar.spark = getglobal( name .. "_Spark" );
    
    -- Set the status bar color
    bar.status:SetStatusBarColor( r, g, b );
    
    -- Set the text color
    bar.text:SetTextColor( 1, 1, 1 );
    
    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateDiscreteBar
--
--  Description:    Create a bar to track a discrete number of values. Note
--                  that the XML is currently hard coded to 5 boxes
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateDiscreteBar( name, unit, count, r, g, b )

    -- Create the bar
    local bar = StatusBars2_CreateBar( name, unit, "StatusBars2_DiscreteBarTemplate" );
    
    -- Save the box count
    bar.boxCount = count;

    -- Initialize the boxes
    local i;
    for i = 1, count do
        local boxName = name .. '_Box' .. i;
        local statusName = name .. '_Box' .. i .. '_Status';
        local box = getglobal( boxName );
        local status = getglobal( statusName );
        box:SetBackdropColor( 0, 0, 0, 0.0 );
        status:SetStatusBarColor( r, g, b );
        status:SetValue( 0 );
    end
    
    return bar;

end;

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateAuraBar
--
--  Description:    Create a bar to display the auras on a unit
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateAuraBar( name, unit )

    -- Create the bar
    local bar = StatusBars2_CreateBar( name, unit, "StatusBars2_AuraBarTemplate" );
    
    -- Initialize the button array
    bar.buttons = {};
    
    -- Set the event handlers
    bar.OnEvent = StatusBars2_AuraBar_OnEvent;
    bar.OnEnable = StatusBars2_AuraBar_OnEnable;
    bar.BarIsVisible = StatusBars2_AuraBar_IsVisible;
    
    -- Register for events
    bar:RegisterEvent( "UNIT_AURA" );
    bar:RegisterEvent( "PLAYER_REGEN_ENABLED" );
    bar:RegisterEvent( "PLAYER_REGEN_DISABLED" );
    if( unit == "target" ) then
        bar:RegisterEvent( "PLAYER_TARGET_CHANGED" );
    end
    
    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_CreateBar
--
--  Description:    Create a status bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_CreateBar( name, unit, template )

    -- Create the bar
    local bar = CreateFrame( "Frame", name, StatusBars2, template );
    bar:Hide( );
    
    -- Set the default methods
    bar.OnEnable = StatusBars2_StatusBar_OnEnable;
    bar.BarIsVisible = StatusBars2_StatusBar_IsVisible;
    
    -- Store the unit
    bar.unit = unit;
    
    -- Save it in the bar collection
    bars[ name ] = bar;
    
    return bar;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_HealthBar_OnEvent
--
--  Description:    Health bar event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_HealthBar_OnEvent( self, event, ... )

    -- Target or Max health changed
    if( event == "UNIT_MAXHEALTH" or event == "PLAYER_TARGET_CHANGED" ) then
        self.status:SetMinMaxValues( 0, UnitHealthMax( self.unit ) );
    
    -- Show when entering combat
    elseif( event == "PLAYER_REGEN_DISABLED" ) then
        self.inCombat = true;
    
    -- Exiting combat
    elseif( event == "PLAYER_REGEN_ENABLED" ) then
        self.inCombat = false;
    
    -- Pet changed
    elseif( event == "UNIT_PET" ) then
        self.status:SetMinMaxValues( 0, UnitHealthMax( self.unit ) );
    end
    
    -- Update
    StatusBars2_UpdateHealthBar( self );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_HealthBar_OnEnable
--
--  Description:    Health bar enable handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_HealthBar_OnEnable( self )

    -- Set the initial minimum and maximumn values
    self.status:SetMinMaxValues( 0, UnitHealthMax( self.unit ) );
    
    -- Update
    StatusBars2_UpdateHealthBar( self );
    
    -- Call the base method
    StatusBars2_StatusBar_OnEnable( self );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdateHealthBar
--
--  Description:    Update a health bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdateHealthBar( self )

    -- Get the current and max health
    local health = UnitHealth( self.unit );
    local maxHealth = UnitHealthMax( self.unit );
    
    -- Update the bar
    StatusBars2_UpdateContinuousBar( self, health, maxHealth );
    
    -- Determine the percentage of health remaining
    local percent = health / maxHealth;
    
    -- Set the bar color based on the percentage of remaining health
    if( percent >= 0.75 ) then
        self.status:SetStatusBarColor( 0, 1, 0 );
    elseif( percent >= 0.50 ) then
        self.status:SetStatusBarColor( 1, 1, 0 );
    elseif( percent >= 0.25 ) then
        self.status:SetStatusBarColor( 1, 0.5, 0 );
    else
        self.status:SetStatusBarColor( 1, 0, 0 );
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_HealthBar_IsVisible
--
--  Description:    Determine if a health bar should be visible
--
-------------------------------------------------------------------------------
--
function StatusBars2_HealthBar_IsVisible( self )

    -- Get the current and max health
    local health = UnitHealth( self.unit );
    local maxHealth = UnitHealthMax( self.unit );
    
    return ( UnitExists( self.unit ) == 1 and UnitIsDeadOrGhost( self.unit ) == nil ) and ( self.inCombat == true or health ~= maxHealth or ( self.unit == "target" and StatusBars2_Settings[ "AlwaysShowTarget" ] == true ) );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_PowerBar_OnEvent
--
--  Description:    Power bar event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_PowerBar_OnEvent( self, event, ... )

    -- Target changed
    if( event == "PLAYER_TARGET_CHANGED" ) then
        StatusBars2_SetPowerBarColor( self );
        self.status:SetMinMaxValues( 0, UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) ) );
        if( self:BarIsVisible( ) == true ) then
            self:Show( );
        else
            self:Hide( );
        end
        StatusBars2_UpdateLayout( );
   
    -- Show the bar when power ticks
    elseif( event == "UNIT_ENERGY" or event == "UNIT_RAGE" or event == "UNIT_MANA" or event == 'UNIT_FOCUS' or event == 'UNIT_RUNIC_POWER' ) then
        if( self:BarIsVisible( ) == true and self:IsVisible( ) ~= 1 ) then
            self:Show( );
        end
        
    -- Update max power
    elseif( ( event == "UNIT_MAXENERGY" or event == "UNIT_MAXRAGE" or event == "UNIT_MAXMANA" or event == "UNIT_MAXFOCUS" or event == 'UNIT_MAXRUNIC_POWER' ) and self.casting ~= true and self.channeling ~= true ) then
        self.status:SetMinMaxValues( 0, UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) ) );
    
    -- Show when entering combat
    elseif( event == 'PLAYER_REGEN_DISABLED' ) then
        self.inCombat = true;
        if( self:BarIsVisible( ) == true and self:IsVisible( ) ~= 1 ) then
            self:Show( );
        end
    
    -- Exiting combat
    elseif( event == 'PLAYER_REGEN_ENABLED' ) then
        self.inCombat = false;
    
    -- Pet changed
    elseif( event == "UNIT_PET" ) then
        self.status:SetMinMaxValues( 0, UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) ) );
        
    -- Unit shapeshifted
    elseif( event == "UNIT_MODEL_CHANGED" and select( 1, ... ) == self.unit ) then
        self.status:SetMinMaxValues( 0, UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) ) );
        StatusBars2_SetPowerBarColor( self );
        StatusBars2_UpdatePowerBar( self );
    
    -- Casting started
    elseif( select( 1, ... ) == self.unit and ( event == "UNIT_SPELLCAST_START" or event == "UNIT_SPELLCAST_CHANNEL_START" or event == "UNIT_SPELLCAST_CHANNEL_UPDATE"  ) and StatusBars2_Settings[ "ShowTargetSpell" ] == true ) then
    
        -- Get spell info
        local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, castID;
        if( event == "UNIT_SPELLCAST_START" ) then
            name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, castID = UnitCastingInfo( self.unit );
        else
            name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitChannelInfo( self.unit );
        end
        
        if( name ~= nil ) then
        
            -- Get the current and max values
            if( event == "UNIT_SPELLCAST_START" ) then
                self.value = GetTime( ) - ( startTime / 1000 );
                self.maxValue = ( endTime - startTime ) / 1000;
                self.castID = castID;
            else
                self.value = ( ( endTime / 1000 ) - GetTime( ) );
                self.maxValue = ( endTime - startTime ) / 1000;
            end
            
            -- Set the bar min, max and current values
            self.status:SetMinMaxValues( 0, self.maxValue );
            self.status:SetValue( self.value );
            
            -- Set the text
            self.text:SetText( name );
            
            -- Show the bar spark
            self.spark:Show( );
            
            -- Set the bar color
            if( event == "UNIT_SPELLCAST_START" ) then
                self.status:SetStatusBarColor( 1.0, 0.7, 0.0 );
            else
                self.status:SetStatusBarColor( 0.0, 1.0, 0.0 );
            end
            
            -- Enter channeling mode
            self.casting = event == "UNIT_SPELLCAST_START";
            self.channeling = event ~= "UNIT_SPELLCAST_START";
            
        end
    
    -- Casting ended
    elseif( event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_INTERRUPTED" or event == "UNIT_SPELLCAST_CHANNEL_STOP" ) then
    
        if( event ~= "UNIT_SPELLCAST_STOP" or select( 4, ... ) == self.castID ) then
            StatusBars2_PowerBar_EndCasting( self );
        end
    
    end
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_PowerBar_OnUpdate
--
--  Description:    Power bar update handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_PowerBar_OnUpdate( self, elapsed )

    -- Casting mode
    if( self.casting == true or self.channeling == true ) then
    
        -- Update the current value
        if( self.casting == true ) then
            self.value = self.value + elapsed;
        else
            self.value = self.value - elapsed;
        end
    
        -- Casting finished
        if( ( self.casting == true and self.value >= self.maxValue ) or ( self.value <= 0 ) ) then
            StatusBars2_PowerBar_EndCasting( self );
        
        -- Casting continuing
        else
            self.status:SetValue( self.value );
            self.spark:SetPoint( "CENTER", self.status, "LEFT", ( self.value / self.maxValue ) * self.status:GetWidth( ), 0 );
            self.percentText:SetText( StatusBars2_Round( self.value / self.maxValue * 100 ) .. "%" );
        end
    
    -- Normal mode
    else
        StatusBars2_UpdatePowerBar( self );
    end

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_PowerBar_EndCasting
--
--  Description:    End spell casting
--
-------------------------------------------------------------------------------
--
function StatusBars2_PowerBar_EndCasting( self )

    -- Exit casting mode
    self.casting = false;
    self.channeling = false;
    self.castID = nil;
    
    -- Reset the min and max values
    self.status:SetMinMaxValues( 0, UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) ) );
    
    -- Hide the bar spark
    self.spark:Hide( );
    
    -- Reset the color
    StatusBars2_SetPowerBarColor( self );
    
    -- Update the bar
    StatusBars2_UpdatePowerBar( self );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_PowerBar_OnEnable
--
--  Description:    Power bar enable handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_PowerBar_OnEnable( self )

    -- Set the initial minimum and maximumn values
    self.status:SetMinMaxValues( 0, UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) ) );
    
    -- Update
    StatusBars2_UpdatePowerBar( self );
    
    -- Call the base method
    StatusBars2_StatusBar_OnEnable( self );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_PowerBar_IsVisible
--
--  Description:    Determine if a power bar should be visible
--
-------------------------------------------------------------------------------
--
function StatusBars2_PowerBar_IsVisible( self )

    -- Get the current and max power
    local power = UnitPower( self.unit, StatusBars2_GetPowerType( self ) );
    local maxPower = UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) );
    
    -- Determine if power is at it's "normal" state
    local powerNormal = ( ( self.powerType == kRage or self.powerType == kRunicPower ) and power == 0 ) or ( power == maxPower );
    
    return ( maxPower > 0 and UnitExists( self.unit ) == 1 and UnitIsDeadOrGhost( self.unit ) == nil ) and ( self.inCombat == true or powerNormal == false or ( self.unit == "target" and StatusBars2_Settings[ "AlwaysShowTarget" ] == true ) );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdatePowerBar
--
--  Description:    Update a power bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdatePowerBar( self )

    -- Get the current and max power
    local power = UnitPower( self.unit, StatusBars2_GetPowerType( self ) );
    local maxPower = UnitPowerMax( self.unit, StatusBars2_GetPowerType( self ) );
    
    -- Update the bar
    StatusBars2_UpdateContinuousBar( self, power, maxPower );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_GetPowerType
--
--  Description:    Get the power type that a power bar is displaying
--
-------------------------------------------------------------------------------
--
function StatusBars2_GetPowerType( self )

    if( self.powerType ~= nil ) then
        return self.powerType;
    else
        return UnitPowerType( self.unit );
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_SetPowerBarColor
--
--  Description:    Set the color of a power bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_SetPowerBarColor( self )

    -- Get the power type
    local powerType = StatusBars2_GetPowerType( self );
    
    -- Get the color based on the power type
    local r, g, b;
    if( powerType == kEnergy ) then
        r, g, b = 1, 1, 0;
    elseif( powerType == kRage ) then
        r, g, b = 1, 0, 0;
    elseif( powerType == kMana ) then
        r, g, b = 0, 0, 1;
    elseif( powerType == kFocus ) then
        r, g, b = 1, 0.5, 0;
    elseif( powerType == kRunicPower ) then
        r, g, b = 0, 0.82, 1;
    else
        r, g, b = 1, 0, 0;
    end
    
    -- Set the bar color
    self.status:SetStatusBarColor( r, g, b );
end


-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_ComboBar_OnEvent
--
--  Description:    Combo bar event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_ComboBar_OnEvent( self, event, ... )

    -- Target changed
    if( event == "PLAYER_TARGET_CHANGED" ) then
        StatusBars2_UpdateDiscreteBar( self, StatusBars2_GetComboPoints( ) );
        
    -- Combo points changed
    elseif( event == "UNIT_COMBO_POINTS" ) then
        local unit = ...;
        if( unit == "player" ) then
            StatusBars2_UpdateDiscreteBar( self, StatusBars2_GetComboPoints( ) );
        end
        
    -- Entering combat
    elseif( event == "PLAYER_REGEN_DISABLED" ) then
        self:Show( );
    
    -- Leaving combat
    elseif( event == "PLAYER_REGEN_ENABLED" ) then
        self:Hide( );
    end
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_ComboBar_OnEnable
--
--  Description:    Combo bar enable handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_ComboBar_OnEnable( self )

    -- Update
    StatusBars2_UpdateDiscreteBar( self, StatusBars2_GetComboPoints( ) );
    
    -- Call the base method
    StatusBars2_StatusBar_OnEnable( self );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_RuneBar_OnEvent
--
--  Description:    Rune bar event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_RuneBar_OnEvent( self, event, ... )

    -- Entering combat
    if( event == "PLAYER_REGEN_DISABLED" ) then
        self:Show( );
        RuneFrame_OnEvent( self, "PLAYER_ENTERING_WORLD" );
    
    -- Leaving combat
    elseif( event == "PLAYER_REGEN_ENABLED" ) then
        self:Hide( );
    end
    
    -- Call the rune frame event handler
    RuneFrame_OnEvent( self, event, ... );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_RuneBar_OnEnable
--
--  Description:    Rune bar enable handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_RuneBar_OnEnable( self )

    -- Update the runes
    RuneFrame_OnEvent( self, "PLAYER_ENTERING_WORLD" );
    
    -- Call the base method
    StatusBars2_StatusBar_OnEnable( self );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_AuraStackBar_OnEvent
--
--  Description:    Aura stack bar event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_AuraStackBar_OnEvent( self, event, ... )

    -- Target changed
    if( event == "PLAYER_TARGET_CHANGED" and self.unit == "target" ) then
        StatusBars2_UpdateDiscreteBar( self, StatusBars2_GetDebuffStack( self.unit, self.aura ) );
        
    -- Entering combat
    elseif( event == "PLAYER_REGEN_DISABLED" ) then
        self:Show( );
    
    -- Leaving combat
    elseif( event == "PLAYER_REGEN_ENABLED" ) then
        self:Hide( );
    
    -- Combat log event
    elseif( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then
    
        -- Get the event type and flags
        local eventType = select( 2, ... );
        local destName = select( 7, ... );
        local flags = select( 8, ... );
        
        -- Only care about events for the unit we are tracking
        if( ( self.unit == "target" and bit.band( flags, COMBATLOG_OBJECT_TARGET ) == COMBATLOG_OBJECT_TARGET ) or ( self.unit == "player" and destName == UnitName( "player" ) ) ) then
        
            -- Look for spell aura events
            if( eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REMOVED" or eventType == "SPELL_AURA_APPLIED_DOSE" or eventType == "SPELL_AURA_REMOVED_DOSE" ) then
            
                -- Look for the aura
                local spellName = select( 10, ... );
                if( string.find( spellName, self.aura ) == 1 ) then
                
                    -- Applied
                    if( eventType == "SPELL_AURA_APPLIED" ) then
                        StatusBars2_UpdateDiscreteBar( self, 1 );
                        
                    -- Removed
                    elseif( eventType == "SPELL_AURA_REMOVED" ) then
                        StatusBars2_UpdateDiscreteBar( self, 0 );
                        
                    -- Dose changed
                    else
                        local amount = select( 13, ... );
                        StatusBars2_UpdateDiscreteBar( self, amount );
                    end
                end
            end
        end
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_AuraBar_OnEvent
--
--  Description:    Aura bar event handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_AuraBar_OnEvent( self, event, ... )

    -- Aurea changed
    if( event == "UNIT_AURA" ) then
        local arg1 = ...;
        if( arg1 == self.unit ) then
            StatusBars2_UpdateAuraBar( self );
        end
        
    -- Target changed
    elseif( event == "PLAYER_TARGET_CHANGED" ) then
        StatusBars2_UpdateAuraBar( self );
    
    -- Entering combat
    elseif( event == 'PLAYER_REGEN_DISABLED' ) then
        self.inCombat = true;
        if( self:BarIsVisible( ) == true and self:IsVisible( ) ~= 1 ) then
            self:Show( );
        end
    
    -- Exiting combat
    elseif( event == 'PLAYER_REGEN_ENABLED' ) then
        self.inCombat = false;
        if( self:BarIsVisible( ) == false ) then
            self:Hide( );
        end
    end;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_AuraBar_OnEnable
--
--  Description:    Aura bar enable handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_AuraBar_OnEnable( self )

    StatusBars2_UpdateAuraBar( self );
    StatusBars2_StatusBar_OnEnable( self );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_AuraBar_IsVisible
--
--  Description:    Determine if an aura bar is visible
--
-------------------------------------------------------------------------------
--
function StatusBars2_AuraBar_IsVisible( self )

    return self.inCombat == true and UnitExists( self.unit ) and UnitIsDeadOrGhost( self.unit ) == nil or ( self.unit == "target" and StatusBars2_Settings[ "AlwaysShowTarget" ] == true );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdateAuraBar
--
--  Description:    Update an aura bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdateAuraBar( self )

    -- Button offset
    local offset = 2;

    -- Hide all the buttons
    for name, button in pairs( self.buttons ) do
        button:Hide( );
    end
    
    -- Buffs
    if( ( self.unit == "player" and StatusBars2_Settings[ "ShowPlayerBuffs" ] == true ) or ( self.unit == "target" and StatusBars2_Settings[ "ShowTargetBuffs" ] == true ) or ( self.unit == "pet" and StatusBars2_Settings[ "ShowPetBuffs" ] == true ) ) then
        local mineOnly = StatusBars2_Settings[ "OnlyShowSelfAuras" ] == true and self.unit ~= "target";
        offset = StatusBars2_ShowAuraButtons( self, "Buff", UnitBuff, MAX_TARGET_BUFFS, mineOnly, offset );
    end
    
    -- Add a space between the buffs and the debuffs
    if( offset > 2 ) then
        offset = offset + StatusBars2_Settings[ "AuraSize" ];
    end
    
    -- Debuffs
    if( ( self.unit == "player" and StatusBars2_Settings[ "ShowPlayerDebuffs" ] == true ) or ( self.unit == "target" and StatusBars2_Settings[ "ShowTargetDebuffs" ] == true ) or ( self.unit == "pet" and StatusBars2_Settings[ "ShowPetDebuffs" ] == true ) ) then
        local mineOnly = StatusBars2_Settings[ "OnlyShowSelfAuras" ] == true and self.unit ~= "player";
        offset = StatusBars2_ShowAuraButtons( self, "Debuff", UnitDebuff, MAX_TARGET_DEBUFFS, mineOnly, offset );
    end
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_ShowAuraButtons
--
--  Description:    Show buff or debuff buttons
--
-------------------------------------------------------------------------------
--
function StatusBars2_ShowAuraButtons( self, auraType, getAuraFunction, maxAuras, mineOnly, offset )

    -- Iterate over the unit auras
    for i = 1, maxAuras do
    
        -- Get the aura
        local name, rank, icon, count, debuffType, duration, expirationTime, isMine = getAuraFunction( self.unit, i );
        
        -- If the aura exists show it
        if( icon ~= nil ) then
        
            -- Determine if the button should be shown
            if( ( isMine == 1 or mineOnly == false ) and ( duration > 0 or StatusBars2_Settings[ "OnlyShowAurasWithDuration" ] == false ) ) then
            
                -- Get the button
                local buttonName = self:GetName( ) .. "_" .. auraType .. "Button" .. i;
                local button = StatusBars2_GetAuraButton( self, i, buttonName, "Target" .. auraType .. "ButtonTemplate", name, rank, icon, count, debuffType, duration, expirationTime, offset );
                
                -- Update the offset
                offset = offset + StatusBars2_Settings[ "AuraSize" ];
                
                -- Show the button
                button:Show( );
            end
        else
            break;
        end
    end
    
    return offset;
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_GetAuraButton
--
--  Description:    Get an aura button for this bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_GetAuraButton( self, id, buttonName, template, auraName, auraRank, auraIcon, auraCount, debuffType, auraDuration, auraExpirationTime, offset )

    -- If the button does not exist create it
    if( self.buttons[ buttonName ] == nil ) then
        self.buttons[ buttonName ] = CreateFrame( "Button", buttonName, self, template );
    end
        
    -- Get the button
    local button = self.buttons[ buttonName ];
    
    -- Set the ID
    button:SetID( id );
    
    -- Set the unit
    button.unit = self.unit;
    
    -- Set the icon
    local buttonIcon = getglobal( buttonName .. "Icon" );
    buttonIcon:SetTexture( auraIcon );
    
    -- Set the cooldown
    local buttonCooldown = getglobal( buttonName .. "Cooldown" );
    if( auraDuration > 0 ) then
        buttonCooldown:Show( );
        CooldownFrame_SetTimer( buttonCooldown, auraExpirationTime - auraDuration, auraDuration, 1 );
    else
        buttonCooldown:Hide( );
    end
    
    -- Set the count
    buttonCount = getglobal( buttonName .."Count" );
    if( auraCount > 1 ) then
        buttonCount:SetText( auraCount );
        buttonCount:Show( );
    else
        buttonCount:Hide( );
    end
   
    -- Set the position
    button:SetPoint( "TOPLEFT", self, "TOPLEFT", offset, -2 );
    
    -- Set the size
    button:SetWidth( StatusBars2_Settings[ "AuraSize" ] );
    button:SetHeight( StatusBars2_Settings[ "AuraSize" ] );
    
    -- If its a debuff set the border size and color
    if( template == "TargetDebuffButtonTemplate" ) then
    
        -- Get debuff type color
        local color = DebuffTypeColor[ "none" ];
        if( debuffType ) then
            color = DebuffTypeColor[ debuffType ];
        end
    
        -- Get the border
        local border = getglobal( buttonName .. "Border" );
        
        -- Set its size and color
        border:SetWidth( StatusBars2_Settings[ "AuraSize" ] + 2 );
        border:SetHeight( StatusBars2_Settings[ "AuraSize" ] + 2 );
        border:SetVertexColor(color.r, color.g, color.b);
    end
       
    return button;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdateContinuousBar
--
--  Description:    Update a continuous bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdateContinuousBar( self, current, max )

    -- If the should not be visible, hide it
    if( self:BarIsVisible( ) == false ) then
        self:Hide( );
        
    -- Otherwise update the bar
    else
    
        -- Show the bar
        if( self:IsVisible( ) ~= 1 ) then
            self:Show( );
        end
        
        -- Set the bar value
        self.status:SetValue( current );
        
        -- Set the text
        self.text:SetText( current .. ' / ' .. max );
        
        -- Set the percent text
        self.percentText:SetText( StatusBars2_Round( current / max * 100 ) .. "%" );
        
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_UpdateDiscreteBar
--
--  Description:    Update a discrete bar
--
-------------------------------------------------------------------------------
--
function StatusBars2_UpdateDiscreteBar( self, current )

    -- Show the frame if necessary
	if( current > 0 ) then
		self:Show( );
    end
    
    -- Update the boxes
    for i = 1, self.boxCount do
    
        -- Get the status bar
        local statusName = self:GetName( ) .. '_Box' .. i .. '_Status';
        local status = getglobal( statusName );
        
        -- If the point exists show it
        if i <= current then
            status:SetValue( 1 );
        else
            status:SetValue( 0 );
        end
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_StatusBar_OnEnable
--
--  Description:    Called when a status bar is enabled
--
-------------------------------------------------------------------------------
--
function StatusBars2_StatusBar_OnEnable( self )

    if( self:BarIsVisible( ) == true ) then
        self:Show( );
    end;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_StatusBar_IsVisible
--
--  Description:    Determine if a status bar should be visible
--
-------------------------------------------------------------------------------
--
function StatusBars2_StatusBar_IsVisible( self )

    -- If in combat and the unit exists show the bar
    return self.inCombat == true and UnitExists( self.unit ) and UnitIsDeadOrGhost( self.unit ) == nil;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_OnDragStart
--
--  Description:    Called when dragging starts
--
-------------------------------------------------------------------------------
--
function StatusBars2_OnDragStart( )

    StatusBars2:StartMoving( );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_OnDragStop
--
--  Description:    Called when dragging stops
--
-------------------------------------------------------------------------------
--
function StatusBars2_OnDragStop( )

    StatusBars2:StopMovingOrSizing( );

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_SetDefaultOptions
--
--  Description:    Set default options
--
-------------------------------------------------------------------------------
--
function StatusBars2_SetDefaultOptions( )

    -- Player health
    if( StatusBars2_Settings.ShowPlayerHealth == nil ) then
        StatusBars2_Settings.ShowPlayerHealth = true;
    end
    
    -- Player power
    if( StatusBars2_Settings.ShowPlayerPower == nil ) then
        StatusBars2_Settings.ShowPlayerPower = true;
    end
    
    -- Player buffs
    if( StatusBars2_Settings.ShowPlayerBuffs == nil ) then
        StatusBars2_Settings.ShowPlayerBuffs = true;
    end
    
    -- Player debuffs
    if( StatusBars2_Settings.ShowPlayerDebuffs == nil ) then
        StatusBars2_Settings.ShowPlayerDebuffs = true;
    end
    
    -- Target health
    if( StatusBars2_Settings.ShowTargetHealth == nil ) then
        StatusBars2_Settings.ShowTargetHealth = true;
    end
    
    -- Target power
    if( StatusBars2_Settings.ShowTargetPower == nil ) then
        StatusBars2_Settings.ShowTargetPower = true;
    end
    
    -- Target buffs
    if( StatusBars2_Settings.ShowTargetBuffs == nil ) then
        StatusBars2_Settings.ShowTargetBuffs = true;
    end
    
    -- Target debuffs
    if( StatusBars2_Settings.ShowTargetDebuffs == nil ) then
        StatusBars2_Settings.ShowTargetDebuffs = true;
    end
    
    -- Pet health
    if( StatusBars2_Settings.ShowPetHealth == nil ) then
        StatusBars2_Settings.ShowPetHealth = ( UnitClass( "player" ) == "Hunter" or UnitClass( "player" ) == "Warlock" );
    end
    
    -- Pet power
    if( StatusBars2_Settings.ShowPetPower == nil ) then
        StatusBars2_Settings.ShowPetPower = ( UnitClass( "player" ) == "Hunter" or UnitClass( "player" ) == "Warlock" );
    end
    
    -- Pet buffs
    if( StatusBars2_Settings.ShowPetBuffs == nil ) then
        StatusBars2_Settings.ShowPetBuffs = ( UnitClass( "player" ) == "Hunter" or UnitClass( "player" ) == "Warlock" );
    end
    
    -- Pet debuffs
    if( StatusBars2_Settings.ShowPetDebuffs == nil ) then
        StatusBars2_Settings.ShowPetDebuffs = ( UnitClass( "player" ) == "Hunter" or UnitClass( "player" ) == "Warlock" );
    end
    
    -- Combo points
    if( StatusBars2_Settings.ShowComboPoints == nil ) then
        StatusBars2_Settings.ShowComboPoints = ( UnitClass( "player" ) == "Rogue" or UnitClass( "player" ) == "Druid" );
    end
    
    -- Runes
    if( StatusBars2_Settings.ShowRunes == nil ) then
        StatusBars2_Settings.ShowRunes = ( UnitClass( "player" ) == "Death Knight" );
    end
    
    -- Deadly Poison
    if( StatusBars2_Settings.ShowDeadlyPoison == nil ) then
        StatusBars2_Settings.ShowDeadlyPoison = UnitClass( "player" ) == "Rogue";
    end
    
    -- Sunder Armor
    if( StatusBars2_Settings.ShowSunderArmor == nil ) then
        StatusBars2_Settings.ShowSunderArmor = UnitClass( "player" ) == "Warrior";
    end
    
    -- Maelstrom Weapon
    if( StatusBars2_Settings.ShowMaelstromWeapon == nil ) then
        StatusBars2_Settings.ShowMaelstromWeapon = UnitClass( "player" ) == "Shaman";
    end
    
    -- Only show self auras
    if( StatusBars2_Settings.OnlyShowSelfAuras == nil ) then
        StatusBars2_Settings.OnlyShowSelfAuras = false;
    end
    
    -- Only show auras with a duration
    if( StatusBars2_Settings.OnlyShowAurasWithDuration == nil ) then
        StatusBars2_Settings.OnlyShowAurasWithDuration = false;
    end
    
    -- Always show target
    if( StatusBars2_Settings.AlwaysShowTarget == nil ) then
        StatusBars2_Settings.AlwaysShowTarget = true;
    end
    
    -- Druid power
    if( StatusBars2_Settings.ShowDruidMana == nil ) then
        StatusBars2_Settings.ShowDruidMana = UnitClass( "player" ) == "Druid";
    end
    
    -- Target spell
    if( StatusBars2_Settings.ShowTargetSpell == nil ) then
        StatusBars2_Settings.ShowTargetSpell = true;
    end
    
    -- Locked
    if( StatusBars2_Settings.Locked == nil ) then
        StatusBars2_Settings.Locked = false;
    end
    
    -- Aura size
    if( StatusBars2_Settings.AuraSize == nil ) then
        StatusBars2_Settings.AuraSize = 16;
    end

end;

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_Options_OnLoad
--
--  Description:    Options frame OnLoad handler
--
-------------------------------------------------------------------------------
--
function StatusBars2_Options_OnLoad( self )

    -- Setup the options panel
    self.name = "StatusBars 2";
    self.okay = StatusBars2_Options_OnOK;
    self.cancel = StatusBars2_Options_OnCancel;
    
    -- Add to the options menu
    InterfaceOptions_AddCategory( self );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_Options_OnOK
--
--  Description:    Called when the OK button is pressed in the options panel
--
-------------------------------------------------------------------------------
--
function StatusBars2_Options_OnOK( )

    -- Update the settings
    StatusBars2_Options_DoDataExchange( true );
    
    -- Update the bar visibility and location
    StatusBars2_UpdateBars( );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_Options_OnCancel
--
--  Description:    Called when the Cancel button is pressed in the options panel
--
-------------------------------------------------------------------------------
--
function StatusBars2_Options_OnCancel( )

    StatusBars2_Options_DoDataExchange( false );
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_Options_DoDataExchange
--
--  Description:    Exchange data between settings and controls
--
-------------------------------------------------------------------------------
--
function StatusBars2_Options_DoDataExchange( save )

    if( save == true ) then
        StatusBars2_Settings[ "ShowPlayerHealth" ] = StatusBars2_Options_PlayerHealthButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPlayerPower" ] = StatusBars2_Options_PlayerPowerButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPlayerBuffs" ] = StatusBars2_Options_PlayerBuffsButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPlayerDebuffs" ] = StatusBars2_Options_PlayerDebuffsButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowTargetHealth" ] = StatusBars2_Options_TargetHealthButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowTargetPower" ] = StatusBars2_Options_TargetPowerButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowTargetBuffs" ] = StatusBars2_Options_TargetBuffsButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowTargetDebuffs" ] = StatusBars2_Options_TargetDebuffsButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPetHealth" ] = StatusBars2_Options_PetHealthButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPetPower" ] = StatusBars2_Options_PetPowerButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPetBuffs" ] = StatusBars2_Options_PetBuffsButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowPetDebuffs" ] = StatusBars2_Options_PetDebuffsButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowComboPoints" ] = StatusBars2_Options_ComboButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowRunes" ] = StatusBars2_Options_RunesButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowDeadlyPoison" ] = StatusBars2_Options_DeadlyPoisonButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowSunderArmor" ] = StatusBars2_Options_SunderArmorButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowMaelstromWeapon" ] = StatusBars2_Options_MaelstromWeaponButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "OnlyShowSelfAuras" ] = StatusBars2_Options_OnlyShowSelfAurasButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "OnlyShowAurasWithDuration" ] = StatusBars2_Options_OnlyShowAurasWithDurationButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "AlwaysShowTarget" ] = StatusBars2_Options_AlwaysShowTargetButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowDruidMana" ] = StatusBars2_Options_DruidManaButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "ShowTargetSpell" ] = StatusBars2_Options_ShowTargetSpellButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "Locked" ] = StatusBars2_Options_LockedButton:GetChecked( ) == 1;
        StatusBars2_Settings[ "AuraSize" ] = StatusBars2_Options_AuraSizeSlider:GetValue( );
    else
        StatusBars2_Options_PlayerHealthButton:SetChecked( StatusBars2_Settings[ "ShowPlayerHealth" ] );
        StatusBars2_Options_PlayerPowerButton:SetChecked( StatusBars2_Settings[ "ShowPlayerPower" ] );
        StatusBars2_Options_PlayerBuffsButton:SetChecked( StatusBars2_Settings[ "ShowPlayerBuffs" ] );
        StatusBars2_Options_PlayerDebuffsButton:SetChecked( StatusBars2_Settings[ "ShowPlayerDebuffs" ] );
        StatusBars2_Options_TargetHealthButton:SetChecked( StatusBars2_Settings[ "ShowTargetHealth" ] );
        StatusBars2_Options_TargetPowerButton:SetChecked( StatusBars2_Settings[ "ShowTargetPower" ] );
        StatusBars2_Options_TargetBuffsButton:SetChecked( StatusBars2_Settings[ "ShowTargetBuffs" ] );
        StatusBars2_Options_TargetDebuffsButton:SetChecked( StatusBars2_Settings[ "ShowTargetDebuffs" ] );
        StatusBars2_Options_PetHealthButton:SetChecked( StatusBars2_Settings[ "ShowPetHealth" ] );
        StatusBars2_Options_PetPowerButton:SetChecked( StatusBars2_Settings[ "ShowPetPower" ] );
        StatusBars2_Options_PetBuffsButton:SetChecked( StatusBars2_Settings[ "ShowPetBuffs" ] );
        StatusBars2_Options_PetDebuffsButton:SetChecked( StatusBars2_Settings[ "ShowPetDebuffs" ] );
        StatusBars2_Options_ComboButton:SetChecked( StatusBars2_Settings[ "ShowComboPoints" ] );
        StatusBars2_Options_RunesButton:SetChecked( StatusBars2_Settings[ "ShowRunes" ] );
        StatusBars2_Options_DeadlyPoisonButton:SetChecked( StatusBars2_Settings[ "ShowDeadlyPoison" ] );
        StatusBars2_Options_SunderArmorButton:SetChecked( StatusBars2_Settings[ "ShowSunderArmor" ] );
        StatusBars2_Options_MaelstromWeaponButton:SetChecked( StatusBars2_Settings[ "ShowMaelstromWeapon" ] );
        StatusBars2_Options_OnlyShowSelfAurasButton:SetChecked( StatusBars2_Settings[ "OnlyShowSelfAuras" ] );
        StatusBars2_Options_OnlyShowAurasWithDurationButton:SetChecked( StatusBars2_Settings[ "OnlyShowAurasWithDuration" ] );
        StatusBars2_Options_AlwaysShowTargetButton:SetChecked( StatusBars2_Settings[ "AlwaysShowTarget" ] );
        StatusBars2_Options_DruidManaButton:SetChecked( StatusBars2_Settings[ "ShowDruidMana" ] );
        StatusBars2_Options_ShowTargetSpellButton:SetChecked( StatusBars2_Settings[ "ShowTargetSpell" ] );
        StatusBars2_Options_LockedButton:SetChecked( StatusBars2_Settings[ "Locked" ] );
        StatusBars2_Options_AuraSizeSlider:SetValue( StatusBars2_Settings[ "AuraSize" ] );
    end
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_GetComboPoints
--
--  Description:    Get the number of combo points for the current player
--
-------------------------------------------------------------------------------
--
function StatusBars2_GetComboPoints( )

    -- Check if the target is dead
    if UnitIsDeadOrGhost( 'target' ) then
        return 0;
    else
        return GetComboPoints( "player", "target" );
    end;
    
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_GetDebuffStack
--
--  Description:    Get the stack size of the specified debuff
--
-------------------------------------------------------------------------------
--
function StatusBars2_GetDebuffStack( unit, debuff )

    local stack = 0;

    -- Iterate over the debuffs on the target
    local i;
    for i = 1, 40 do
    
        -- Get the debuff
        local name, rank, texture, count = UnitDebuff( unit, i );
        
        -- Check the name
        if( name == debuff ) then
            stack = count;
            break;
        elseif( name == nil ) then
            break;
        end;
    end
    
    return stack;

end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_Trace
--
--  Description:    Trace a message to the console
--
-------------------------------------------------------------------------------
--
function StatusBars2_Trace( message )
    DEFAULT_CHAT_FRAME:AddMessage( message );
end

-------------------------------------------------------------------------------
--
--  Name:           StatusBars2_Round
--
--  Description:    Round a numer
--
-------------------------------------------------------------------------------
--
function StatusBars2_Round( x )
    return floor( x + 0.5 );
end
